{
open Parser
exception Error

let keywords = Hashtbl.create 17

let lower s =
  assert (String.length s > 0) ;
  let r = String.copy s in
  r.[0] <- Char.lowercase s.[0] ;
  r

let check_keyword s =
  try Hashtbl.find keywords s with Not_found -> IDENT s

and add_keyword s k =
  Hashtbl.replace keywords s k ;
  Hashtbl.replace keywords (lower s) k ;

and rem_keyword s =
  Hashtbl.remove keywords s ;
  Hashtbl.remove keywords (lower s)

let () =
  add_keyword "Let" LET ;
  add_keyword "In" IN ;
  add_keyword "Fun" FUN ;
  add_keyword "Fix" FIX ;
  add_keyword "Ifz" IFZ ;
  add_keyword "Then" THEN ;
  add_keyword "Else" ELSE ;
  add_keyword "Rec" REC ;
  add_keyword "Ref" REF ;
  ()
}

rule token = parse
| [' ''\t''\n'] { token lexbuf }
| ['A'-'Z''a'-'z']  ['A'-'Z''a'-'z''0'-'9''_'] * as ident
   { check_keyword ident }
| ['0'-'9']+ as x
   { INT (int_of_string x) }
| "->" { ARROW }
| "="  { EQUAL }
| ":=" { SET }
| "!"  { GET }
| ";"  { SEQ }
| "("  { OPAR }
| ")"  { CPAR }
| "+"  { ADD }
| "-"  { SUB }
| "*"  { MUL }
| "/"  { DIV }
| ":"  { COLON }
| ";;"  { SEMISEMI }
| eof  { EOF }
| ""   { raise Error }

and sync = shortest _* ( ";;" | eof) { () }
{
let get_token = token
}
